package juniq;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import java.util.concurrent.*;
import java.security.*;
import javax.swing.table.*;

public class Generate extends JFrame
{
    JTextField scriptname = new JTextField("/tmp/juniq-script.sh");
    JButton genbutton = new JButton("Generate Script");
    JSpinner minsize = new JSpinner();
    JCheckBox delsymlinks = new JCheckBox("", true);
    JCheckBox delemptydirs = new JCheckBox();
    JCheckBox delnonfiles = new JCheckBox("", true);
    JCheckBox showprogress = new JCheckBox("", true);
    JTextArea action = new JTextArea("rm -f $'$1'");
    JComboBox algorithm = new JComboBox();

    public static final long serialVersionUID = 1001;

    JUniq juniq;

    public Generate(JUniq juniq)
    {
	super("Script Generator");

	this.juniq=juniq;
	minsize.setValue(new Integer(-1));

	algorithm.addItem("Keep shortest pathname");
	algorithm.addItem("Keep longest pathname");

	Container me = getContentPane();

	me.setLayout(new GridLayout(8,2));

	me.add(new JLabel("Output script name"));
	me.add(scriptname);

	me.add(new JLabel("Delete action: "));
	me.add(action);

	me.add(new JLabel("Show progress: "));
	me.add(showprogress);

	me.add(new JLabel("Don't delete files smaller than"));
	me.add(minsize);

	me.add(new JLabel("Delete symlinks"));
	me.add(delsymlinks);

	//	me.add(new JLabel("Delete empty directories"));
	//	me.add(delemptydirs);

	me.add(new JLabel("Delete non files/non dirs"));
	me.add(delnonfiles);

	me.add(new JLabel("Selection strategy"));
	me.add(algorithm);

	me.add(new JLabel(" "));
	me.add(genbutton);

	genbutton.addActionListener(new ActionListener() 
	    { public void actionPerformed(ActionEvent e) { generate(); }});

	setSize(400,250);
	setVisible(false);
    }

    public void generate()
    {
	try {
	    generateReal();
	    setVisible(false);
	} catch (IOException ex) {
	    System.out.println("unexpected exception: "+ex);
	}
    }

    public void generateReal() throws IOException
    {
	FileWriter outs = new FileWriter(scriptname.getText());
	Progress p;
	int pos;

	outs.write("# JUniq-generated removal script\n");
	outs.write("# \n");
	outs.write("# symlinks\n");

	if (delsymlinks.isSelected())
	    {
		p = new Progress("Symlinks  ", outs, juniq.symLinks.size());
		pos=0;
		p.start();
		for (File f: juniq.symLinks)
		    {
			removefile(outs, f);
			p.progress(pos++);
		    }

		p.end();
	    }
	else
	    outs.write("# (skipped)\n");

	outs.write("# \n");
	outs.write("# non files, non directories\n");
	if (delsymlinks.isSelected())
	    {
		p = new Progress("Non-Files ", outs, juniq.nonFiles.size());
		pos = 0;
		p.start();
		for (File f: juniq.nonFiles)
		    {
			removefile(outs, f);
			p.progress(pos++);
		    }
		p.end();
	    }
	else
	    outs.write("# (skipped)\n");

	outs.write("# \n");
	outs.write("# Duplicate files\n");
	outs.write("# \n");

	long minfilesize = ((Integer) minsize.getValue()).intValue();
	pos = 0;

	p = new Progress("Duplicates", outs, juniq.dupFileSets.size());
	p.start();

	for (FileSet fset: juniq.dupFileSets)
	    {
		if (fset.fileLength < minfilesize)
		    continue;

		int survivor = picksurvivor(fset);
		outs.write("# Survivor: "+fset.files.get(survivor).getAbsolutePath()+"\n");

		for (int i=0;i<fset.files.size();i++)
		    {
			if (i==survivor)
			    continue;

			removefile(outs, fset.files.get(i));
		    }
		outs.write("# \n");

		p.progress(pos++);
	    }
	p.end();

	outs.write("# End of file\n");
	outs.flush();
	outs.close();
    }

    public void removefile(FileWriter outs, File f) throws IOException
    {
	String n = action.getText();
	n = n.replace("$0",f.getAbsolutePath());
	n = n.replace("$1",escape(f.getAbsolutePath()));
	outs.write(n+"\n");
    }

    public String escape(String s)
    {
	StringBuffer sb = new StringBuffer();

	for (int i=0;i<s.length();i++)
	    {
		char c = s.charAt(i);
		if (c==' ' || c=='.' || c=='/' || Character.isLetterOrDigit(c))
		    sb.append(c);
		else
		    sb.append(String.format("\\x%2x", c&0xff));
	    }
	return sb.toString();
    }

    public int picksurvivor(FileSet fset) throws IOException
    {
	switch (algorithm.getSelectedIndex())
	    {
	    case 0:
		return picksurvivor_shortestpathname(fset);
	    case 1:
		return picksurvivor_longestpathname(fset);
	    }

	assert(false);
	return 0;
    }

    public int picksurvivor_longestpathname(FileSet fset) throws IOException
    {
	int survivor = -1;
	int longestpath = 0;

	for (int i=0;i<fset.files.size();i++)
	    {
		int pathlength = fset.files.get(i).getCanonicalPath().length();
		if (pathlength > longestpath)
		    {
			survivor=i;
			longestpath = pathlength;
		    }
	    }
	return survivor;
    }

    public int picksurvivor_shortestpathname(FileSet fset) throws IOException
    {
	int survivor = -1;
	int shortestpath = Integer.MAX_VALUE;

	for (int i=0;i<fset.files.size();i++)
	    {
		int pathlength = fset.files.get(i).getCanonicalPath().length();
		if (pathlength <shortestpath)
		    {
			survivor=i;
			shortestpath = pathlength;
		    }
	    }
	return survivor;
    }

    class Progress
    {
	int lastpercent;
	String s;
	FileWriter outs;
	int max;

	public Progress(String s, FileWriter outs, int max)
	{
	    this.s=s;
	    this.outs=outs;
	    this.max=max;
	}

	public void progress(int value) throws IOException
	{
	    int percent = value*100/max;

	    if (percent>lastpercent)
		{
		    percent(percent);
		    lastpercent=percent;
		}
	}

	public void start() throws IOException
	{
	    percent(0);
	}

	public void end() throws IOException
	{
	    percent(100);
	    outs.write("echo\n");
	}

	void percent(int p) throws IOException
	{
	    if (!showprogress.isSelected())
		return;

	    outs.write(String.format("echo -en \"\\r%s [%3d%%]\"\n", s, p));
	}
    }

 
}
